<?php

use Supportingcast\ScPlans;

add_action('admin_menu', 'sc_settings_page');

function sc_settings_page() {
	add_submenu_page(
		'options-general.php', // top level menu page
		'Supporting Cast', // title of the settings page
		'Supporting Cast', // title of the submenu
		'manage_options', // capability of the user to see this page
		'supporting-cast', // slug of the settings page
		'sc_admin_page_html' // callback function when rendering the page
	);
	add_action('admin_init', 'sc_settings_init');
}

function sc_settings_init() {
	add_settings_section(
		'supporting-cast-general', // id of the section
		'General', // title to be displayed
		'', // callback function to be called when opening section, currently empty
		'supporting-cast' // page on which to display the section
	);



	register_setting(
		'supporting-cast', // option group
		'sc-token'
	);
	register_setting(
		'supporting-cast', // option group
		'sc-member-source'
	);
	register_setting(
		'supporting-cast', // option group
		'sc-plans'
	);



	add_settings_field(
		'sc-token', // id of the settings field
		'Token', // title
		'sc_setting_api_token', // callback function
		'supporting-cast', // page on which settings display
		'supporting-cast-general', // section on which to show settings
		[
            'field' => 'sc-token',
            'label' => 'Token',
            'el' => 'textarea',
            'help' => (SC_API_DOMAIN_BASE != '.supportingcast.fm' ? 'Connecting to api' . SC_API_DOMAIN_BASE : '')
        ]
	);
	add_settings_field(
		'sc-member-source', // id of the settings field
		'User Source', // title
		'sc_form_element_render', // callback function
		'supporting-cast', // page on which settings display
		'supporting-cast-general', // section on which to show settings,
		['field' => 'sc-member-source',
         'label' => 'Member Source',
         'el' => 'select',
         'options' => ['wordpress' => 'Wordpress', 'memberful' => 'Memberful']
         ]
	);
	add_settings_field(
		'sc-plans', // id of the settings field
		'User Plan(s)', // title
		'sc_form_plans_render', // callback function
		'supporting-cast', // page on which settings display
		'supporting-cast-general' // section on which to show settings
	);
}

function sc_form_element_render($args)
{
	$value = get_option($args['field']);
	if ($args['el'] === 'textarea') {
		echo "<textarea style=\"width:300px; height: 200px\" id=\"{$args['field']}\" name=\"{$args['field'] }\">{$value}</textarea>";
	} else if ($args['el'] === 'input') {
		echo "<input id=\"{$args['field'] }\" name=\"{$args['field'] }\" value=\"{$value}\">";
	} else if ($args['el'] === 'select') {
		echo "<select id=\"{$args['field'] }\" name=\"{$args['field'] }\">";
		foreach ($args['options'] as $option => $label)
		{
			$selected = $option === $value ? 'selected ' : '';
			echo "<option {$selected}value=\"{$option}\">{$label}</option>";
		}
	}
	if (!empty($args['help'])) {
		echo '<div class="sc-help">' . $args['help'] . '</div>';
	}
}

function sc_setting_api_token() {

	$value = get_option('sc-token');

	echo "<textarea style=\"width:300px; height: 200px\" id=\"sc-token\" name=\"sc-token\" class='sc-blur-field'>{$value}</textarea><br>";

	if (SC_API_DOMAIN_BASE != '.supportingcast.fm') {
		echo '<div class="sc-help">Connecting to api' . SC_API_DOMAIN_BASE .'</div>';
	}

	if (isset($_GET['oauth_status'])) {

	    $noticeType = ($_GET['oauth_status'] == 'success' ? 'notice-success' : 'notice-error');
	    echo "<div class='notice $noticeType'>" . \Supportingcast\ApiTokenOauth::getStatusMessage($_GET['oauth_status']) . '</div>';
    }

	?>
    <button id='sc_oauth_redirect_button' name='SC_OAUTH_REDIRECT' value='1' onclick="return sc_oauth_start()">Generate new API token</button>

    <script type="application/javascript">
        function sc_oauth_start() {
            var domain = window.prompt("Please enter your Supporting Cast subdomain:", "?.supportingcast.fm");

            console.debug(domain);

            if (
                (domain == null)
                || (domain.trim() == '')
                || (domain == '?.supportingcast.fm')
                || (domain == '.supportingcast.fm')
                || (domain == 'www.supportingcast.fm')
                || (domain == 'api.supportingcast.fm')
                ) {
                document.location = "/wp-admin/options-general.php?page=supporting-cast&oauth_status=bad_domain";
                return false;
            }

            document.getElementById('sc_oauth_redirect_button').value = domain;

            return true;

        };
    </script>
    <?

}

function sc_form_plans_render()
{
	try {
		$sc_plans = new ScPlans();
		$sc_plans->fetch();
		$sc_plans = $sc_plans->getFreePublicPlans();
	} catch (\Throwable $e) {
	    echo $e->getMessage();
	    return;
    }

    $plans = get_option('sc-plans', []);
	$id = 0;

    ?>
    <div class="mbrsource-wrapper">
        <div class="hidden mbrsource mbrsource-wordpress">
            <select class="sc-plan-input" disabled name="sc-plans[<?= $id ?>]">
		        <?php
		        echo "<option value=\"\">None</option>";
		        foreach($sc_plans as $sc_plan) {
			        $selected = ($plans[$id] == $sc_plan->id) ? 'selected ' : '';
			        echo "<option {$selected} value=\"{$sc_plan->id}\">{$sc_plan->name}</option>";
		        }
		        ?>
            </select>
        </div>
        <div class="hidden mbrsource mbrsource-memberful">
            <table>
            <tr>
                <th>Memberful Plan</th><th>Supporting Cast Plan</th>
            </tr>
            <?php

                $products = get_option('memberful_subscriptions', []);
                foreach ($products as $id => $product)
                {
                ?>
                    <tr>
                        <td>
                            <?= $product['name'] ?> (<?= $id ?>)
                        </td>
                        <td>
                            <select class="sc-plan-input" disabled name="sc-plans[<?= $id ?>]">
	                            <?php
	                                echo "<option value=\"\">None</option>";
                                    foreach($sc_plans as $sc_plan) {
                                        $selected = ($plans[$id] == $sc_plan->id) ? 'selected ' : '';
	                                    echo "<option {$selected} value=\"{$sc_plan->id}\">{$sc_plan->name}</option>";
                                    }
	                            ?>
                            </select>
                        </td>
                    </tr>
                <?php
                }
                ?>
            </table>
        </div>
    </div>
    <div style="margin-top: 10px" class="sc-help">Only <b>Free</b> plans can be used in Wordpress integrations.</div>

    <script>
        changeScMbrSource = function(ev) {
            let target = (ev && ev.target) ? ev.target : document.getElementById('sc-member-source'),
                source = target.value,
                mbsources = document.getElementsByClassName('mbrsource');

            for (let x = 0; x < mbsources.length; x++) {
                let hidden = 'add',
                    disabled = true,
                    inputs = mbsources[x].getElementsByClassName('sc-plan-input');
                if (mbsources[x].classList.contains('mbrsource-' + source)) {
                    hidden = 'remove';
                    disabled = false;
                }
                mbsources[x].classList[hidden]('hidden');

                for (let y = 0; y < inputs.length; y++) {
                    inputs[y].disabled = disabled;
                }

            }
        }
        changeScMbrSource();
        document.getElementById('sc-member-source').addEventListener('change', changeScMbrSource);
    </script>
    <?php
}

function sc_admin_page_html()
{
	?>
	<div class="wrap">
		<h1>Supporting Cast</h1>
		<form method="POST" action="options.php">
			<?php settings_fields('supporting-cast'); ?>
			<?php do_settings_sections('supporting-cast'); ?>
			<?php submit_button(); ?>
		</form>
	</div>
	<?php
}
